/*********************************************************************

  proptree.cpp

  Author:  Russell Morris
  Release: February 26, 1997

  TPropTreeDialog - implementation of what I like to call "property
                    trees".  See the readme file for documentation info.

*********************************************************************/
#include <owl/pch.h>
#include <owl/propsht.h>
#include <owl/treewind.h>

#include "proptree.h"
#include "proptree.rh"

DEFINE_RESPONSE_TABLE1( TPropTreeDialog, TDialog )
  EV_BN_CLICKED( IDAPPLY, CmApply ),
  EV_BN_CLICKED( IDC_RESET, CmReset ),
  EV_BN_CLICKED( IDHELP, CmHelp ),
  EV_BN_CLICKED( IDCANCEL, CmCancel ),
  EV_BN_CLICKED( IDOK, CmOk ),

  EV_TVN_SELCHANGING( IDC_PROPTREE, EvSelChanging ),
  EV_TVN_BEGINLABELEDIT( IDC_PROPTREE, EvBeginLabelEdit ),
  EV_TVN_ENDLABELEDIT( IDC_PROPTREE, EvEndLabelEdit ),
  EV_TVN_GETDISPINFO( IDC_PROPTREE, EvGetDispInfo ),
  EV_TVN_ITEMEXPANDED( IDC_PROPTREE, EvItemExpanded ),
  EV_TVN_ITEMEXPANDING( IDC_PROPTREE, EvItemExpanding ),
  EV_TVN_SELCHANGED( IDC_PROPTREE, EvSelChanged ),
  EV_TVN_SETDISPINFO( IDC_PROPTREE, EvSetDispInfo ),
  EV_TVN_KEYDOWN( IDC_PROPTREE, EvKeyDown ),
END_RESPONSE_TABLE;


/**********************************************************************
  class TPaneInfo - used to hold state info about panes.
**********************************************************************/

/*
  TPaneInfo default constructor

  Purpose: Facilitate use of TPaneInfo with the TArrayAsVector
           template, which requires a default constructor.  We will
           not use any TPaneInfo's created like this, because we just
           zero out all members here.
*/
TPaneInfo::TPaneInfo()
{
  id = 0;
  dirty = false;
}


/*
  TPaneInfo main constructor

  Purpose: Used to create a valid TPaneInfo object to be stored in the
           paneInfoArray.

  Parameters:
    newId : the 32bit unique identifier of the pane this TPaneInfo is to
    represent.
*/
TPaneInfo::TPaneInfo( uint32 newId )
{
  id = newId;
  dirty = false;
}


/*
  TPaneInfo destructor

  Purpose: clean up after class.
*/
TPaneInfo::~TPaneInfo()
{
}


/*
  operator "=" for TPaneInfo

  Purpose: Assigns "this" pane to the "other".
*/
TPaneInfo&
TPaneInfo::operator = (const TPaneInfo other)
{
  id = other.id;
  dirty = other.dirty;
  return *this;
}


/*
  operator "==" for TPaneInfo

  Purpose: This function is required by TArrayAsVector.  It considers two
           TPaneInfo's to be equal if they are for the same pane (ie their
           id members are the same).

  Return value: returns nonzero if the two are equal, zero if not.
*/
int
TPaneInfo::operator == (const TPaneInfo other)
{
  return id == other.id;
}



/*********************************************************************
  class TPropTreedialog - base class that defines the behaviour of a
                          Property Tree.  See documentation for a more
                          in-depth (and better) description.
*********************************************************************/

/*
  Constructor for TPropTreeDialog

  Purpose: Creates buttons, the treeview, the pane info text control, and
           the placement rectangle used to position TPropTreePanes.

  Parameters:
    parent : A pointer to the TWindow that is our parent
    resId : As always, the resource id of our dialog box template
    module : A pointer (usually NULL) that points to the TModule that
             holds our dialog template.
*/
TPropTreeDialog::TPropTreeDialog(TWindow* parent,
                                 TResId resId,
                                 TModule* module)
:
    TDialog(parent, resId, module),
    paneInfoArray(10,0,30)
{
  tree = new TTreeWindow( this, IDC_PROPTREE );

  placement = new TStatic( this, IDC_PLACEMENT );
  paneInfo  = new TStatic( this, IDC_PANEDESC );

  Apply = new TButton( this, IDAPPLY );
  Reset = new TButton( this, IDC_RESET );
}


/*
  Destructor for TPropTreeDialog

  Purpose: Cleans up after this instance.  Calls Destroy() to kill the
           actual window element.
*/
TPropTreeDialog::~TPropTreeDialog()
{
  Destroy();
}


/*
  CmApply

  Purpose: Tell each pane to apply any changes made.
  Pre: The user has pressed the Apply button
  Post: Each pane is asked to apply its changes.  If any pane has problems
        with this, they are selected so the user can fix whatever is wrong.

  Comment: CmApply() does not mark the panes as clean - you need to call
           PaneUnchanged() from your TPropTreePane descendant in response
           to successfull calls to its Apply().
*/
void
TPropTreeDialog::CmApply()
{
  TPropTreePane* pane;
  TPaneInfoArrayIter iter( paneInfoArray );

  while( iter )
  {
    if( iter.Current().dirty )
    {
      TPaneInfo info = iter.Current();
      pane = GetPane( info.id );

      // If the pane's Apply() results in error, go ahead and abort and
      // select the bitchy pane so the user can do whatever needs
      // to be done.
      //
      if( pane && pane->Apply() )
      {
        SelectPane( info.id );
        ::MessageBeep( MB_ICONHAND );
        return;
      }
    }
    iter++;
  }

  // deactivate the Apply and Reset buttons
  //
  EvaluateApplyStatus();
  EvaluateResetStatus();
}


/*
  EvaluateApplyStatus

  Purpose: Determine whether the apply button shoule be enabled or not.
           This means that if any of the panes is dirty, it should be
           enabled, otherwise it should be disabled.
  Pre: none
  Post: The Apply button is enabled or disabled

*/
void
TPropTreeDialog::EvaluateApplyStatus()
{
  TPaneInfoArrayIter iter( paneInfoArray );
  bool state=false;

  // Look thru each pane to see if it's dirty
  //
  while( iter )
  {
    // This little op will leave state with a value of true if
    // any panes are dirty
    //
    state = state || iter.Current().dirty;

    iter++;
  }

  // If any panes are dirty, leave the Apply button active
  //
  Apply->EnableWindow(state);
}


/*
  CmReset

  Purpose: Resets the current pane in response to the user pressing
           the "Reset" button.
  Pre: the user has pressed the "Reset" button
  Post: the current pane is reset
*/
void
TPropTreeDialog::CmReset()
{
  ResetPane( GetCurrentPane()->GetId() );
}


/*
  ResetPane

  Purpose: Function tells a pane to reset itself, then marks the pane as
           clean.  Then checks to see whether Apply/Reset should be
           enabled.
  Pre: called by CmReset
  Post: pane with Id paneId is reset

  Parameters:
    paneId : The unique id of a pane in the tree
*/
void
TPropTreeDialog::ResetPane( uint32 paneId )
{
  // Tell the current pane to undo all changes since the Apply button
  // was last pressed.
  //
  GetPane(paneId)->Reset();

  // Remember that the pane is now clean
  //
  int index = paneInfoArray.Find( TPaneInfo(paneId) );
  if( index == INT_MAX )
    return;
  TPaneInfo& info = paneInfoArray[index];
  info.dirty=false;

  // Can't reset this pane anymore
  //
  EvaluateResetStatus();

  // Was this the only dirty pane?
  //
  EvaluateApplyStatus();
}


/*
  CanReset

  Purpose: Determines whether a given pane (via paneId) is eligible
           to be Reset
  Pre: a pane with paneId exists in the tree
  Post: see return value

  Return value: Returns true if the pane is dirty (ie can be reset),
                and false otherwise.
*/
bool
TPropTreeDialog::CanReset( uint32 paneId )
{
  // Find the info about the pane with paneId
  //
  int index = paneInfoArray.Find( TPaneInfo(paneId) );

  if( index == INT_MAX )
    return false;

  // Return whether it's dirty (can be reset)
  //
  TPaneInfo info = paneInfoArray[index];
  return info.dirty;
}


/*
  EvaluateResetStatus

  Purpose: Enables/Disables the Reset button depending on the state
           of the current pane.
  Pre: none
  Post: If the current pane can be reset, the "Reset" button is enabled.
        Otherwise it is disabled.
*/
void
TPropTreeDialog::EvaluateResetStatus()
{
  bool enable;
  TPropTreePane* pane;

  // Get the current pane
  //
  pane = GetCurrentPane();

  // Determine its ability to reset
  //
  if(!pane)
    enable = false;
  else
    enable = CanReset( pane->GetId() );

  // And enable the "Reset" button accordingly
  //
  Reset->EnableWindow( enable );
}


/*
  CmHelp

  Purpose:  Called in response to a user pressing the "Help" button.
  Pre: the user has pressed the "Help" button.
  Post: The current pane is told to display help info about itself
*/
void
TPropTreeDialog::CmHelp()
{
  // The help button was pressed - notify the active pane of the event.
  // Help is handled by the active pane.
  GetCurrentPane()->Help();
}


/*
  CmCancel

  Purpose: Override the Cancel functionality so we can ask panes if it
           is ok to cancel the dialog.
  Pre: The cancel button has been pressed
  Post: If all panes are ok with it, the dialog is cancelled
*/
void
TPropTreeDialog::CmCancel()
{
  // Ask each pane if it's ok to cancel the dialog (and thus
  // ignore all changes since the last apply
  //
  TPropTreePane* pane;
  TPaneInfoArrayIter iter( paneInfoArray );

  while( iter )
  {
    if( iter.Current().dirty )
    {
      TPaneInfo info = iter.Current();
      pane = GetPane( info.id );

      // If the pane's QueryCancel() results in error, go ahead and abort and
      // select the bitchy pane so the user can do whatever needs
      // to be done.
      //
      if( pane && pane->QueryCancel() )
      {
        SelectPane( info.id );
        ::MessageBeep( MB_ICONHAND );
        return;
      }
    }
    iter++;
  }

  // We'll return from the function by this point if there was an error.
  // So if we get here, we know no panes had a problem with cancelling.
  //
  CloseWindow(IDCANCEL);
}


/*
  CmOk

  Purpose: In response to an Ok button press, we need to tell each pane to
           apply changes.
  Pre: the OK button has been pressed.
  Post: If any panes had problems applying data, we return from this
        function without closing the window
*/
void
TPropTreeDialog::CmOk()
{
  TPropTreePane* pane;
  TPaneInfoArrayIter iter( paneInfoArray );

  while( iter )
  {
    if( iter.Current().dirty )
    {
      TPaneInfo info = iter.Current();
      pane = GetPane( info.id );

      // If the pane's Apply() results in error, go ahead and abort and
      // select the bitchy pane so the user can do whatever needs
      // to be done.
      //
      if( pane && pane->Apply() )
      {
        SelectPane( info.id );
        ::MessageBeep( MB_ICONHAND );
        return;
      }
    }
    iter++;
  }
  CloseWindow( IDOK );
}


/*
  SetupWindow

  Purpose: Gotta do a little bit of setup before we display the dialog.
  Pre: none - called by TWindow
  Post: the Apply and Reset buttons are disabled.
*/
void
TPropTreeDialog::SetupWindow()
{
  TDialog::SetupWindow();

  // AddPane is only called after this function, so it is
  // guarunteed that there are no panes yet.
  Apply->EnableWindow(false);
  Reset->EnableWindow(false);
}




/*
  AddPane

  Purpose: Add a pane to the tree.
  Pre: pane is a pointer to a valid TPropTreePane
  Post: If no errors are encountered, the pane is added to the tree.

  Parameters:
    pane : A valid pointer to the TPropTreePane to be added to the tree.
           This pane must be new'd, but the window element does not have
           to be Create()'d yet.
    parent : A TTreeNode object that is to be the pane's parent in the
             tree.
    text : A pointer (optionally null) that points to the title of the
           pane that will be displayed in the tree
    image : The image index of the pane - this can be just zero if you
            aren't using imagelists.
    selImage : the selected image index of the pane - this can be just
               zero if you aren't using imagelists.

  Return value: Returns a copy of the TTreeNode representing the pane
                you just added so you can use it as the parent of other
                panes.
*/
TTreeNode
TPropTreeDialog::AddPane( TPropTreePane* pane,
                          TTreeNode parent,
                          const char far* text,
                          int image,
                          int selImage )
{
  char buff[100];

  // If they didn't give us a title, use the pane's window title
  //
  if( !text )
    pane->GetWindowText( buff, 99 );
  else
    strcpy( buff, text );

  // Now do the actual work of inserting the pane
  //
  return InsertPane( pane, parent, buff, image, selImage );
}


/*
  InsertPane

  Purpose: This private function actually does the work of AddPane.
  Pre: Called by AddPane to add a new pane to the tree
  Post: the new pane is added to the tree

  Parameters:
    pane : A valid pointer to the TPropTreePane to be added to the tree.
           This pane must be new'd, but the window element does not have
           to be Create()'d yet.
    parent : A TTreeNode object that is to be the pane's parent in the
             tree.
    text : A pointer (optionally null) that points to the title of the
           pane that will be displayed in the tree
    image : The image index of the pane - this can be just zero if you
            aren't using imagelists.
    selImage : the selected image index of the pane - this can be just
               zero if you aren't using imagelists.

  Return value: Returns a copy of the TTreeNode representing the pane
                you just added so you can use it as the parent of other
                panes.
*/
TTreeNode
TPropTreeDialog::InsertPane( TPropTreePane* pane,
                             TTreeNode parent,
                             const char far* text,
                             int image,
                             int selImage )
{
  // Create the node to add
  //
  TTreeNode paneNode( *tree,
                       text,
                       image,
                       selImage );

  // store a pointer to the pane as the item data
  //
  paneNode.SetItemData( (uint32)pane );

  if( !paneInfoArray.Add(TPaneInfo(pane->GetId())) )
    return TTreeNode( *tree, (HTREEITEM)NULL );

  // Add the node to the tree, creating another node (with
  // a valid HTREEITEM) which we can pass back to the caller.
  //
  TTreeNode addedNode( parent.AddChild(paneNode) );

  tree->Update();

  // Return this TTreeNode so it can be used as the parent of others.
  //
  return addedNode;
}


/*
  SelectPane

  Purpose: Finds and selects a pane given its id.
  Pre: none
  Post: the pane with id is selected (if it exists)

  Parameters:
    id : the id of a pane in the tree
*/
void
TPropTreeDialog::SelectPane( uint32 id )
{
  TTreeNode node = GetPaneNode( id );
  if( !node )
    return;
  node.SelectItem( TTreeNode::Caret );
}

/*
  GetPane

  Purpose: Gets a pointer to the TPropTreePane via its id.  Wraps
           internal implementation by PaneFromId
  Pre: A pane with id should exist
  Post: see return value

  Parameters:
    id : the id of a pane in the tree

  Return value: Returns a TPropTreePane pointer with id.
*/
TPropTreePane*
TPropTreeDialog::GetPane( uint32 id )
{
  return PaneFromId( id );
}


/*
  PaneFromId

  Purpose: Internal function used to find a pane given its id.  First gets
           the TTreeNode with id "id", then gets the item data which is
           a pointer to the TPropTreePane.
  Pre: none
  Post: see Return value

  Parameters:
    id : An id of a pane in the tree

  Return value:  Returns a pointer to the pane with id "id", or NULL
                 if no pane with that id exists.
*/
TPropTreePane*
TPropTreeDialog::PaneFromId( uint32 id )
{
  TPropTreePane* pane;

  // Find the node that matches to this id
  //
  TTreeNode node = GetPaneNode( id );
  if( !node )
    return NULL;

  node.GetItem(); //Make sure that everything is up to date
  node.GetItemData( (uint32)pane ); // itemData is a pointer to the pane
  return pane;
}


/*
  GetPaneNode

  Purpose: Gets a pane's TTreeNode given its id, calling the internal
           function FindNodeWithId
  Pre: a pane with id "id" exists in the tree
  Post: see return value

  Parameters:
    id : An id of a pane in the tree

  Return value: Returns a TTreeNode representing the pane with id "id"
*/
TTreeNode
TPropTreeDialog::GetPaneNode( uint32 id )
{
  TTreeNode root( *tree );
  TTreeNode paneNode( *tree );

  FindNodeWithId( id, root, paneNode );
  return paneNode;
}


/*
  FindNodeWithId

  Purpose: Private recursive function that walks the tree until it
           finds a pane with id "id".
  Pre: none
  Post: After this function has recursed over the nodes in the tree and
        found the correct one, it will be stored in the "paneNode"
        TTreeNode& reference

  Parameters:
    id : the id of the pane we are looking for
    parent : the current parent of the children we're looking at right now
    paneNode : A reference to the node which we will assign the correct
               node to.
*/
void
TPropTreeDialog::FindNodeWithId( uint32 id,
                                 TTreeNode parent,
                                 TTreeNode& paneNode )
{
  TPropTreePane* pane;
  int i; //to check for child items

  // if this isn't the root, check to see what its id is
  //
  if( (HTREEITEM)parent != TVI_ROOT )
  {
    parent.GetItem();
    parent.GetItemData( (uint32)pane );
    if( pane && pane->GetId() == id )
    {
      paneNode = parent;
      return;
    }

  parent.GetHasChildren( i );
  if( i != 1 )
    // If there aren't any children, go ahead and return
    //
    return;
  }

  // Walk parent's child list.
  //
  for ( TTreeNode& child = parent.GetChild();
        child;
        child = child.GetNextSibling() )
  {
    // Go ahead and check to see if child is the pane's node
    //
    child.GetItemData( (uint32)pane );
    if( pane->GetId() == id )
    {
      paneNode = child;
      return;
    }

    // If this node has children, recurse over them, too
    //
    child.GetHasChildren( i );
    if( i )
    {
      FindNodeWithId( id, child, paneNode );
    }
  }
}


/*
  SelectNextPane

  Purpose: Finds and selects a node given a TNextCode.
  Pre: A node is selected
  Post: The node returned from the GetNextItem() call is selected.

  Comments: This differs from SelectPane() because SelectPane() matches
            a given pane's id to its node and selects it.  SelectNextPane()
            finds a node via a TNextCode and activates that node's pane.
*/
void
TPropTreeDialog::SelectNextPane( TTreeNode::TNextCode nc )
{
  TTreeNode node = GetSelection();
  TTreeNode next = node.GetNextItem( nc );

  if( !next ) //make sure we didn't get junk back
    return;

  // now select "next"
  //
  next.SelectItem( TTreeNode::Caret );
}


/*
  GetCurrentPane

  Purpose: Gets a pointer to the pane that is currently selected
  Pre: there is a selected pane
  Post: see return value

  Return value: Returns the TPropTreePane associated with the currently
                selected node.
*/
TPropTreePane*
TPropTreeDialog::GetCurrentPane()
{
  TPropTreePane* pane;
  TTreeNode node = tree->GetSelection();
  if( (HTREEITEM)node == NULL )
    return 0;

  // Get the item data, which is a pointer to the pane
  //
  node.GetItemData( (uint32)(pane), true );

  return pane;
}


/*
  PaneChanged

  Purpose: Called by panes when something about them changes.  We need
           to mark them as dirty and enable the Apply and Reset buttons.
  Pre: "id" is the id of a pane in the tree
  Post: This pane is marked as dirty.
*/
void
TPropTreeDialog::PaneChanged( uint32 id )
{
  int i = paneInfoArray.Find( TPaneInfo(id) );
  if( i == INT_MAX )
    return;
  paneInfoArray[i].dirty = true;

  // Activate buttons
  //
  EvaluateApplyStatus();
  EvaluateResetStatus();
}


/*
  PaneUnchanged

  Purpose: Called by a pane when it wants to mark itself as un-dirty.
           Again we need to check the states of the Apply and Reset
           buttons
  Pre: a pane with id "id" exists in the tree
  Post: The pane is marked as clean, the Apply and Reset buttons are
        enabled accordingly

  Parameters:
    id : An id of a pane in the tree
*/
void
TPropTreeDialog::PaneUnchanged( uint32 id )
{
  int i = paneInfoArray.Find( TPaneInfo(id) );
  if( i == INT_MAX )
    return;
  paneInfoArray[i].dirty = false;

  // Check buttons' status
  //
  EvaluateApplyStatus();
  EvaluateResetStatus();
}


//
// These functions are here to allow access to the TTreeWindow
//     --==============================================--
// See TTreeWindow documentation for these functions
//
bool
TPropTreeDialog::EvBeginLabelEdit( TTwDispInfoNotify& )
{
  return false;
}


void
TPropTreeDialog::EvEndLabelEdit( TTwDispInfoNotify& )
{
}


void
TPropTreeDialog::EvGetDispInfo( TTwDispInfoNotify& )
{
}


void
TPropTreeDialog::EvItemExpanded( TTwNotify& )
{
}


bool
TPropTreeDialog::EvItemExpanding( TTwNotify& )
{
  return false;
}


void
TPropTreeDialog::EvSelChanged( TTwNotify& )
{
}


void
TPropTreeDialog::EvSetDispInfo( TTwDispInfoNotify& )
{
}


void
TPropTreeDialog::EvKeyDown( TTwKeyDownNotify& )
{
}


/*******************************************************/

/*
  EvSelChanging

  Purpose: Takes care of activating the selected node's pane.  If either
           pane has objections, don't let the selection change.
  Pre: called when a new node is selected in the tree.
  Post: If all goes well, the new pane is given focus and the old one
        is hidden

  Parameters:
    nm : A reference to the poorly wrapped TTwNotify structure

  Return value: Returns false to allow the change, true to disallow.
*/
bool
TPropTreeDialog::EvSelChanging( TTwNotify& nm )
{
  TTreeItem oldItem( nm.itemOld );
  TTreeItem newItem( nm.itemNew );

  TPropTreePane* oldPane = (TPropTreePane*)oldItem.GetItemData();
  TPropTreePane* newPane = (TPropTreePane*)newItem.GetItemData();

  // Give everyone a chance to setup if necessary
  //
  if( !PreSwitchPanes( nm ) &&
      !DoSwitchPanes( oldPane, newPane ) )
  {
    PanesSwitched( oldPane, newPane );
    return false;
  }
  return true;
}


/*
  PreSwitchPanes

  Purpose: Give derived classes a chance to veto pane selection change
           depending on circumstances that may arise there.
  Pre: We have just recieved a EvSelChanging notification
  Post: Upon returning false, DoSwitchPanes and possibly PanesSwitched
        will be called

  Parameters:
    nm : A TTwNotify class with info about what happened

  Return value: Return "true" to stop the change, false otherwise
*/
bool
TPropTreeDialog::PreSwitchPanes( TTwNotify& )
{
  return false;
}


/*
  PanesSwitched

  Purpose: Give derived classes a chance to act when two panes are
           switched.
  Pre: Called after two panes are switched.
  Post: The derived class has had a chance to respond to this event.

  Parameters:
    oldPane : A pointer to the pane that is about to be hidden
    newPane : A pointer to the pane that is about to be shown.

  Comments:  Note that you cannot veto the switching of the panes here.
             You should do that in your derived pane classes.
             Also, either of these parameters may be null.
*/
void
TPropTreeDialog::PanesSwitched( TPropTreePane* /*oldPane*/,
                                TPropTreePane* /*newPane*/ )
{
}


/*
  DoSwitchPanes

  Purpose: The actual work of switching panes is done here.  This function
           determines whether both panes are ok with the switch and then
           switches them.
  Pre: Called in response to EvSelChanging()
  Post: If all is cool, the panes are switched.  If oldPane doesn't want
        to lose focus, it is kept active and newPane->SetActive is never
        called.  If newPane doesn't want to be selected, oldPane is
        reselected, resulting in a call to its SetActive().

  Parameters:
    oldPane : A pointer (may be null) to the pane losing focus
    newPane : A pointer (may be null) to the pane gaining focus

  Return value: Returns false if the panes are to be switched, true
                otherwise.
*/
bool
TPropTreeDialog::DoSwitchPanes( TPropTreePane* oldPane,
                                TPropTreePane* newPane )
{
  if(oldPane)
  {
    // Give oldPane a chance to veto the switch
    //
    if( oldPane->KillActive() )
    {
      MessageBeep( MB_ICONHAND );
      EnsureVisible( oldPane->GetId() );
      return true;
    }

    // Hide the window  (no redraw == no flicker)
    //
    oldPane->SetWindowPos( HWND_TOP,
                           TRect( 0, 0, 0, 0 ),
                           SWP_HIDEWINDOW|SWP_NOACTIVATE|SWP_NOSIZE|SWP_NOREDRAW );
  }

  if(newPane)
  {
    // If the window hasn't been created, create it now
    //
    if( !newPane->GetHandle() )
      newPane->Create();

    // Give newPane a chance to veto selection...
    //
    if( newPane->SetActive() && oldPane )
    {
      MessageBeep( MB_ICONHAND );
      SelectPane( oldPane->GetId() );
      return true;
    }

    // Move it into position...
    //
    newPane->SetWindowPos( HWND_TOP,
                           TRect( placement->Attr.X, placement->Attr.Y, 0, 0 ),
                           SWP_HIDEWINDOW|SWP_NOACTIVATE|SWP_NOSIZE );

    // Make sure we can see the pane; set the info text about the pane,
    // and check the status of the Reset button
    //
    newPane->ShowWindow(SW_SHOW);
    paneInfo->SetText( newPane->GetHelpText().c_str() );
    EvaluateResetStatus();
  }

  return false; //Success
}


/*
  EnsureVisible

  Purpose: Makes sure that a given pane's node is visible to the user.
           This is just a redefinition of TTreeNode::EnsureVisible made
           to accept paneId's instead of HTREEITEMs
  Pre: paneId is the id of a pane in the tree
  Post: If it exists, the node is guarunteed to be visible to the user.

  Parameters:
    paneId : the id of a pane in the tree

  Return value: returns the value returned by node.EnsureVisible(),
                which does the standard "false" on success and "true" on
                failure.
*/
bool
TPropTreeDialog::EnsureVisible( uint32 paneId )
{
  TTreeNode node = GetPaneNode(paneId);
  if(node)
    return node.EnsureVisible();

  return true;
}


/*
  ExpandPane

  Purpose: Another simple redefinition of a TTreeNode function rewritten
           to select nodes via their ids
  Pre: none
  Post: The node is expanded according to the expand flag

  Parameters:
    id : the id of a pane in the tree
    flag : the expand code (see treeview documentation)
           TVE_EXPAND
           TVE_COLLAPSE
           TVE_TOGGLE

  Return value: "false" on success, "true" otherwise
*/
bool
TPropTreeDialog::ExpandPane( uint32 id, uint32 flag )
{
  TTreeNode node = GetPaneNode( id );
  if( !node )
    return false;
  return node.ExpandItem( flag );
}


/*
  RemoveChildItem

  Purpose: Remove a child node from the node with id paneId. This is
           for use only within TPropTreePanes.  It is used to
           dynamically remove child nodes.
  Pre: childToRemoveId is the id of a pane which you want to remove
  Post: the pane with childToRemoveId is removed

  Parameters:
    childToRemoveId : the id of the pane that you want to remove from the
                      tree.
*/
void
TPropTreeDialog::RemoveChildItem( uint32 childToRemoveId )
{
  // Get the node that we need to remove and its parent node
  //
  TTreeNode node = GetPaneNode( childToRemoveId );
  TTreeNode parent = node.GetParent();
  int paneInfoIndex;

  // Remove the node from the tree.  The pane pointer will be disposed of
  // when the TWizPropTreeDialog closes.
  //
  if(!node.Delete())
    MessageBox("Oh shit");

  // We also need to take out the reference to this node from our
  // paneInfoArray.
  //
  paneInfoIndex = paneInfoArray.Find( TPaneInfo(childToRemoveId) );
  paneInfoArray.RemoveEntry( paneInfoIndex );
}



/*
  AddChildItem

  Purpose: Add a child node to the node with id paneId.  This is for
           use only from within TWizPropTreePane.  It is used to
           dynamically add child nodes.
  Pre: paneId must be the unique id of the parent you are adding to;
       newChildPane must be a valid pointer to a TPropTreePane.
  Post: The new child is added to the node with "paneId"

  Parameters:
    paneId : the id of the pane that this new child pane will be added to.
    newChildPane : A pointer to a TPropTreePane that will be added
    title : a pointer (optionally null) to the text to be displayed in
            the tree for this pane
    image : The image index for this pane's node
    selImage : The selected image index for this pane's node
*/
void
TPropTreeDialog::AddChildItem( uint32 paneId,
                               TPropTreePane* newChildPane,
                               const char far *title,
                               int image,
                               int selImage )
{
  // Get our parent node to add to...
  //
  TTreeNode parent = GetPaneNode( paneId );

  // Insert the pane...
  //
  AddPane( newChildPane, parent, title, image, selImage );
}



