//----------------------------------------------------------------------------
//  Project scenedit
//  Allgood Software
//  Copywrite 1995
//
//  SUBSYSTEM:    scenedit.apx Application
//  FILE:         proptreepage.cpp
//  AUTHOR:       Russell Morris
//
//  OVERVIEW
//  ~~~~~~~~
//  Source file for implementation of TPropTreePage (TPropertyPage).
//
//----------------------------------------------------------------------------
#include <owl/pch.h>
#include <owl/propsht.h>
#include <owl/treewind.h>

#include "proptreepage.h"
#include "sessions.rh"

DEFINE_RESPONSE_TABLE1( TPropTreePage, TPropertyPage )
  EV_TVN_SELCHANGING( IDC_PROPTREE, EvSelChanging ),
  EV_TVN_BEGINLABELEDIT( IDC_PROPTREE, EvBeginLabelEdit ),
  EV_TVN_ENDLABELEDIT( IDC_PROPTREE, EvEndLabelEdit ),
  EV_TVN_GETDISPINFO( IDC_PROPTREE, EvGetDispInfo ),
  EV_TVN_ITEMEXPANDED( IDC_PROPTREE, EvItemExpanded ),
  EV_TVN_ITEMEXPANDING( IDC_PROPTREE, EvItemExpanding ),
  EV_TVN_SELCHANGED( IDC_PROPTREE, EvSelChanged ),
  EV_TVN_SETDISPINFO( IDC_PROPTREE, EvSetDispInfo ),
  EV_TVN_KEYDOWN( IDC_PROPTREE, EvKeyDown ),
END_RESPONSE_TABLE;

// TPaneInfo - used to hold state information about panes
//
TPaneInfo::TPaneInfo()
{
  id = 0;
  dirty = false;
}


TPaneInfo::TPaneInfo( uint32 newId )
{
  id = newId;
  dirty = false;
}


TPaneInfo::~TPaneInfo()
{
}


TPaneInfo&
TPaneInfo::operator = (const TPaneInfo other)
{
  id = other.id;
  dirty = other.dirty;
  return *this;
}


int
TPaneInfo::operator == (const TPaneInfo other)
{
  return id == other.id;
}


// TPropTreePage
//
TPropTreePage::TPropTreePage(TPropertySheet* parent,
                             TResId resId,
                             const char far* title,
                             TResId iconRes,
                             TModule* module)
:
    TPropertyPage(parent, resId, title, iconRes, module),
    paneInfoArray(10,0,30)
{
  tree = new TTreeWindow( this, IDC_PROPTREE );
  placement = new TStatic( this, IDC_PLACEMENT );
}


TPropTreePage::~TPropTreePage()
{
  Destroy();
}


void
TPropTreePage::SetupWindow()
{
  TPropertyPage::SetupWindow();
}


int
TPropTreePage::Apply( TNotify far& )
{
  TPropTreePane* pane;
  TPaneInfoArrayIter iter( paneInfoArray );

  while( iter )
  {
    if( iter.Current().dirty )
    {
      TPaneInfo& info = iter.Current();
      pane = PaneFromId( info.id );

      // If the pane's Apply() results in error, go ahead and abort
      //
      if( pane && pane->Apply() )
        return PSNRET_INVALID_NOCHANGEPAGE;

      // The pane applied changes successfully - it's clean now
      //
      info.dirty = false;
    }
    iter++;
  }

  // All pane's were Ok, so ask ourself
  //
  return DoApply();
}


int
TPropTreePage::DoApply()
{
  return PSNRET_NOERROR;
}


bool
TPropTreePage::KillActive( TNotify far& )
{
  // We need to notify every pane that the entire page is about to lose
  // focus.  It would be easier to just notify the current pane, but some
  // pane's might not have been selected yet and thus have not had the
  // chance to verify or complain about any of their data.
  //
  TPropTreePane* pane;
  TPaneInfoArrayIter iter( paneInfoArray );

  while( iter )
  {
    if( iter.Current().dirty )
    {
      pane = PaneFromId( iter.Current().id );

      // If the pane doesn't want to lose focus, go ahead and abort
      //
      if( pane && pane->KillActive() )
        return true;
    }
    iter++;
  }

  // All pane's were Ok, so ask ourself
  //
  return DoKillActive();
}


bool
TPropTreePage::DoKillActive()
{
  return false;
}


int
TPropTreePage::SetActive(TNotify&)
{
  TPropTreePane* pane = GetCurrentPane();

  if( pane && pane->SetActive() == -1 )
    return -1;
  return DoSetActive();
}


int
TPropTreePage::DoSetActive()
{
  return 0;
}


void
TPropTreePage::Reset(TNotify&)
{
  DoReset();
}


void
TPropTreePage::DoReset()
{
}


void
TPropTreePage::Help(TNotify&)
{
  // You might have other controls on your page besides to property tree,
  // so put the help-calling code inside DoHelp().
  //
  DoHelp();
}


void
TPropTreePage::DoHelp()
{
  // Calls the pane's help function, which should bring up a help page for
  // that pane.  If you want to call a default help page for the entire
  // page (ie. not for each pane - you might have other controls on your
  // page) override this function.
  //
  GetCurrentPane()->Help();
}


int
TPropTreePage::WizBack(TNotify& nf)
{
  // Just check with the panes to see if it's alright to leave
  //
  if( KillActive(nf) || DoWizBack() == -1)
    return -1;
  return 0;
}


int
TPropTreePage::DoWizBack()
{
  return 0;
}


int
TPropTreePage::WizNext(TNotify& nf)
{
  // Just check with the panes to see if it's alright to leave
  //
  if( KillActive(nf) || DoWizBack() == -1)
    return -1;
  return 0;
}


int
TPropTreePage::DoWizNext()
{
  return 0;
}


bool
TPropTreePage::WizFinish(TNotify& nf)
{
  if( Apply(nf) || DoWizFinish() == -1 )
    return true;
  return false;
}


bool
TPropTreePage::DoWizFinish()
{
  return 0;
}


bool
TPropTreePage::QueryCancel(TNotify&)
{
  return DoQueryCancel();
}


bool
TPropTreePage::DoQueryCancel()
{
  return 0;
}


//
// Add panes to the tree
//
TTreeNode
TPropTreePage::AddPane( TPropTreePane* pane,
                        TTreeNode parent,
                        const char far* text,
                        int image,
                        int selImage )
{
  char buff[100];

  if( !text )
    pane->GetWindowText( buff, 99 );
  else
    strcpy( buff, text );

  TTreeNode paneNode( *tree,
                      buff,
                      image,
                      selImage );

  // store a pointer to the pane as the item data
  //
  paneNode.SetItemData( (uint32)pane );

  // Add a TPaneInfo to hold the pane's dirty info
  //
  paneInfoArray.Add( TPaneInfo( pane->id ) );



  TTreeNode addedNode( parent.AddChild(paneNode) );
  tree->Update();

  return addedNode;
}


void
TPropTreePage::SelectPane( uint32 id , TTreeNode::TNextCode nc)
{
  TTreeNode node = NodeFromId( id );
  if( !node )
    return;

  node.SelectItem( nc );
}


TPropTreePane*
TPropTreePage::PaneFromId( uint32 id )
{
  TPropTreePane* pane;

  TTreeNode node = NodeFromId( id );
  node.GetItem(); //Make sure that everything is up to date
  node.GetItemData( (uint32)pane ); // itemData is a pointer to the pane
  return pane;
}


TTreeNode
TPropTreePage::NodeFromId( uint32 id )
{
  TTreeNode root( *tree );
  TTreeNode paneNode( *tree );

  FindNodeWithId( root, id, paneNode );
  return paneNode;
}


void
TPropTreePage::FindNodeWithId( TTreeNode parent, uint32 id, TTreeNode& paneNode )
{
  TPropTreePane* pane;
  int i; //to check for child items

  if( (HTREEITEM)parent != TVI_ROOT )
  {
    parent.GetItem();
    parent.GetItemData( (uint32)pane );
    if( pane->id == id )
    {
      paneNode = parent;
      return;
    }
  if( !parent.GetHasChildren( i ) )
    return;
  }

  for ( TTreeNode& child = parent.GetChild();
        child;
        child = child.GetNextSibling() )
  {
    // Go ahead and check to see if child is the pane's node
    //
    child.GetItemData( (uint32)pane );
    if( pane->id == id )
    {
      paneNode = child;
      return;
    }

    // If this node has children, call recurse over them, too
    //
    child.GetHasChildren( i );
    if( i == 1 ) // i should'nt be I_IMAGECALLBACK, but I'm paranoid
    {
      FindNodeWithId( child, id, paneNode );
    }
  }
}

void
TPropTreePage::SelectNextPane( TTreeNode::TNextCode nc )
{
  TTreeNode node = GetSelection();
  node.SelectItem( nc );
}


TPropTreePane*
TPropTreePage::GetCurrentPane()
{
  TPropTreePane* pane;
  TTreeNode node = tree->GetSelection();
  if( (HTREEITEM)node == 0 )
    return 0;
  node.GetItemData( (uint32)(pane), true );
  return pane;
}


void
TPropTreePage::PaneChanged( uint32 id )
{
  int i = paneInfoArray.Find( TPaneInfo(id) );
  if( i == INT_MAX )
    return;
  paneInfoArray[i].dirty = true;

  PageChanged();
}


void
TPropTreePage::PaneUnchanged( uint32 id )
{
  int i = paneInfoArray.Find( TPaneInfo(id) );
  if( i == INT_MAX )
    return;
  paneInfoArray[i].dirty = false;
}


void
TPropTreePage::PageChanged()
{
  Parent->SendMessage(PSM_CHANGED, (WPARAM)(HWND)*this, 0L);
}


void
TPropTreePage::PageUnchanged()
{
  Parent->SendMessage(PSM_UNCHANGED, (WPARAM)(HWND)*this, 0L);
}


bool
TPropTreePage::EvBeginLabelEdit( TTwDispInfoNotify& )
{
  return false;
}


void
TPropTreePage::EvEndLabelEdit( TTwDispInfoNotify& )
{
}


void
TPropTreePage::EvGetDispInfo( TTwDispInfoNotify& )
{
}


void
TPropTreePage::EvItemExpanded( TTwNotify& )
{
}


bool
TPropTreePage::EvItemExpanding( TTwNotify& )
{
  return false;
}


void
TPropTreePage::EvSelChanged( TTwNotify& )
{
}


bool
TPropTreePage::EvSelChanging( TTwNotify& nm )
{
  TTreeItem oldItem( nm.itemOld );
  TPropTreePane* oldPane = (TPropTreePane*) oldItem.GetItemData();
  if(oldPane)
  {
    // Give oldPane a chance to veto the switch
    //
    if( oldPane->KillActive() )
      return true;

    // Move it out of the way  (no redraw == no flicker)
    //  * I haven't seen screen resolution above 1600x1200...
    //
    oldPane->SetWindowPos( HWND_TOP,
                           TRect( 2000, 2000, 0, 0 ),
                           SWP_HIDEWINDOW|SWP_NOACTIVATE|SWP_NOSIZE|SWP_NOREDRAW );
  }

  TTreeItem newItem( nm.itemNew );
  TPropTreePane* newPane = (TPropTreePane*)newItem.GetItemData();
  if(newPane)
  {
    // If the window hasn't been created, create it now
    //
    if( !newPane->GetHandle() )
      newPane->Create();

    // Give newPane a chance to veto selection...
    //
    if( newPane->SetActive() )
      return true;

    // Move it into position...
    //
    newPane->SetWindowPos( HWND_TOP,
                           TRect( placement->Attr.X, placement->Attr.Y, 0, 0 ),
                           SWP_HIDEWINDOW|SWP_NOACTIVATE|SWP_NOSIZE );
    newPane->ShowWindow(SW_SHOW);
  }
  return false;
}




void
TPropTreePage::EvSetDispInfo( TTwDispInfoNotify& )
{
}


void
TPropTreePage::EvKeyDown( TTwKeyDownNotify& )
{
}


bool
TPropTreePage::EnsureVisible( uint32 paneId )
{
}


bool
TPropTreePage::ExpandPane( uint32 id, uint32 flag )
{
  TTreeNode node = NodeFromId( id );
  if( !node )
    return false;
  return node.ExpandItem( flag );
}

