/*********************************************************************

  proptreepane.cpp

  Author:  Russell Morris
  Release: February 26, 1997

  TPropTreePane - Implementation of a "pane" (or a page) in a property
                  tree. 

*********************************************************************/
#include <owl/pch.h>

#include <owl/propsht.h>

#include "proptreepane.h"

#include "proptreepage.h"


/*
  TPropTreePane Constructor

  Purpose: Sets up data members according to parameters passed.

  Parameters:
    parent: A pointer to the TPropTreeDialog parent of this pane.
    uniqueId: A uint32 value uniquely identifying this pane in the
              tree.
    helpText: The text to be displayed in the pane info box below
              the tree.
    resId: A resource identifier of this pane's dialog box template
    module: A pointer (optionally null) pointing to the TModule that
            contains the dialog template with resource id resId.
*/
TPropTreePane::TPropTreePane( TPropTreeDialog* parent,
                              uint32 uniqueId,
                              const char far* text,
                              TResId resId,
                              TModule* module )
:
    TDialog(parent, resId, module), id(uniqueId),
    helpText(text)
{
}


/*
  Destructor for TPropTreePane

  Purpose: Calls Destroy() to clean up all the window elements, and then
           lets C++ do its job.
*/
TPropTreePane::~TPropTreePane()
{
  Destroy();
}


/*
  KillActive

  Purpose: Called when TPropTreeDialog wants to give focus to some other
           pane.
  Pre: none
  Post: Returns true or false (see return value below).

  Return value: Returns "true" to refuse to relinquish the focus to another
                pane, "false" otherwise.

  Comment: In derived classes, you'll want to override this function and
           perform checking to see if all the data your pane needs from
           the user has been entered, its valid/correct, etc...
*/
bool
TPropTreePane::KillActive()
{
  return false; // return true to keep page active
}


/*
  Apply

  Purpose: Called whenever the user presses the Apply or Ok button.  You
           should redefine this function in your class to store whatever
           data has been entered.
  Pre: The Ok or Apply button has been pressed.
  Post: All the data in this pane has been stored or processed in some way
        (it's up for your derived class to figure out what to do)

  Return value:  If you find there is something wrong with the data in
                 your pane which prevents you from fully processing
                 everything, return "true" to stop the Apply process.
                 If no problems are encountered, just return "false".

  Comment:   If you return "true" to stop the Apply process, make sure that
           you tell the user why you are stopping the process, what is
           wrong with the data they entered, and what they need to fix.
             Also, just before you return from Apply you'll almost always
           want to call PaneUnchanged() to tell TPropTreeDialog that you
           are now clean.
*/
bool
TPropTreePane::Apply()
{
  return false; // return true to veto apply
}


/*
  SetActive

  Purpose: Called by TPropTreeDialog to ask us we will accept the focus
  Pre: The previously active pane has returned "false" from KillActive().
  Post: See return value

  Return value: Returns false to accept the focus, "true" otherwise.

  Comments:  If you return "true" to refuse to accept focus, it is your
             responsibility to bitch at the user and tell them what's
             wrong.  If you do return true, the pane that was previously
             active will still be active (ie no change will take place).

*/
bool
TPropTreePane::SetActive()
{
	return false; //
}


/*
  Reset

  Purpose: Called by TPropTreeDialog when the user presses the
           "Reset" button.  You should reset all data to the state it
           was before the user started entering stuff.
  Pre: User has pressed "Reset" button
  Post: The data in the pane is restored to its unaltered state.
*/
void
TPropTreePane::Reset()
{
}


/*
  Help

  Purpose: Called when the user presses the "Help" button.  You need to
           redefine this virtual function in your derived class if you
           want to provide pane-specific help.
  Pre: The user has pressed the help button.
  Post: You display help for this pane.
*/
void
TPropTreePane::Help()
{
  MessageBox("There is no help topic associated with this pane",
             "Error", MB_OK|MB_ICONEXCLAMATION );
}


/*
  QueryCancel

  Purpose: When the user presses the "Cancel" or "Close" button,
           TPropTreeDialog asks each pane, via QueryCancel, if it's
           Ok to cancel the dialog.
  Pre: The user has pressed the "Cancel" button
  Post: See return value

  Return value: Return "false" to allow the cancelation, "true" to stop
                the process.  If you return "true", your pane will be
                given focus (resulting in a call to your SetActive()).
                You should then tell the user what's wrong and how to
                fix it.
*/
bool
TPropTreePane::QueryCancel()
{
  return false; // return true to prevent cancel operation
}




/*
  PaneChanged

  Purpose: Inform the TPropTreeDialog that we've changed.
  Pre: Our data has changed somehow via user interaction
  Post: TPropTreeDialog "dirties" the pane and enables the Apply and
        Reset buttons.
*/
void
TPropTreePane::PaneChanged()
{
  GetPropTree()->PaneChanged( id );
}


/*
  PaneUnchanged

  Purpose: Inform the TPropTreeDialog that we do not want to be
           considered "dirty" anymore.
  Pre: none
  Post: TPropTreeDialog marks this pane as clean and then disables the
        "Reset" button.  If this pane was the only pane that was dirty,
        it also disables the "Apply" button.

  Comments: You will most likely call this function in response to
            Apply(), after which you will most likely want to be
            considered clean.
*/
void
TPropTreePane::PaneUnchanged()
{
  GetPropTree()->PaneUnchanged( id );
}



