/*********************************************************************

  proptree.h

  Author:  Russell Morris
  Release: February 26, 1997

  Class definition of TPropTreeDialog

*********************************************************************/
#if !defined(proptree_h)              // Sentry, use file only if it's not already included.
#define proptree_h

#include <owl/propsht.h>
#include <owl/treewind.h>
#include <owl/imagelst.h>
#include <owl/static.h>
#include <classlib/arrays.h>
#include <owl/button.h>

//
// Keep info about each pane the property tree contains
//
class TPaneInfo {
  public:
    TPaneInfo();
    TPaneInfo( uint32 id );

    ~TPaneInfo();

    TPaneInfo& operator = (const TPaneInfo other);
    int operator == (const TPaneInfo other);

    // state info about pane
    //
    uint32  id;
    bool    dirty;

    // A pointer to the pane
    //
//    TPropTreePane* pane;
};


typedef TArrayAsVector< TPaneInfo > TPaneInfoArray;
typedef TArrayAsVectorIterator< TPaneInfo > TPaneInfoArrayIter;

// Forward declaration for TPropTreePane.  We can't include it's class def
// before TPropTreeDialog because it relies on the class definition of
// TPropTreeDialog.
//
class TPropTreePane;

class TPropTreeDialog : public TDialog {
  public:
    TPropTreeDialog( TWindow* parent,
                     TResId resId,
                     TModule* module = 0);
    virtual ~TPropTreeDialog();

  public:


    void SetupWindow();

    // Add a pane to the tree
    //

    TTreeNode AddPane( TPropTreePane* pane,
                       TTreeNode parent,
                       const char far* text,
                       int i = 0,
                       int j = 0);

    // Allow panes to add child items to themselves.  For example, if you
    // ask the user if they want to do something, you may want to add
    // different panes depending on whether they choose "yes" or "no".
    // Don't add child items unless you know that they will be used.
    // If the user changes their mind, you can remove child items by
    // calling RemoveChildItem();
    //
    void AddChildItem( uint32 paneId,
                               TPropTreePane* newChildPane,
                               const char far* title,
                               int image,
                               int selImage );
    void RemoveChildItem( uint32 childToRemoveId );

    // Pane change notifications
    //
    virtual void PaneChanged( uint32 id );
    virtual void PaneUnchanged( uint32 id );

    // Pane gettin'
    //
    TPropTreePane* GetPane( uint32 id );
    TPropTreePane* GetCurrentPane();

    // Find a pane's node
    //
    TTreeNode GetPaneNode( uint32 id );

    // Wrap manipulation of nodes
    //
    void SelectPane( uint32 id );
    void SelectNextPane( TTreeNode::TNextCode nc = TTreeNode::NextVisible );
    bool ExpandPane(uint32 flag, uint32 paneId);

    // Functions called by pane's in a wizard dialog
    //
    void  WizardPaneUpdated( uint paneId );

  protected:
    // Respond to TreeView events.  Allows derived classes to get
    // ahold of the TreeWindow notifications.
    //
    virtual bool  EvBeginLabelEdit( TTwDispInfoNotify& );
    virtual void  EvEndLabelEdit( TTwDispInfoNotify& );
    virtual void  EvGetDispInfo( TTwDispInfoNotify& );
    virtual void  EvItemExpanded( TTwNotify& );
    virtual bool  EvItemExpanding( TTwNotify& );
    virtual void  EvSelChanged( TTwNotify& );
    virtual bool  EvSelChanging( TTwNotify& );
    virtual void  EvSetDispInfo( TTwDispInfoNotify& );
    virtual void  EvKeyDown( TTwKeyDownNotify& );


    // Button press functions
    //
    void  CmApply();
    void  CmReset();
    virtual void  CmHelp();
    virtual void  CmCancel();
    virtual void  CmOk();

    // Reset pane's
    //
    bool CanReset( uint32 paneId );
    void ResetPane( uint32 paneId );

    // Give derived classes a chance to veto seletion
    //
    virtual bool PreSwitchPanes( TTwNotify& );

    // About to switch two panes... this does any setup necessary by
    // derived classes.  The calling function (EvSelChanging) then calls
    // DoSwitchPanes, which does the actual work.
    //
    virtual void PanesSwitched( TPropTreePane *oldPane,
                                TPropTreePane *newPane );

    //
    // == These are all TTreeWindow functions ==
    // Use them to manipulate the underlying treeview control
    // A couple were modified slightly to facilitate their use
    // in TPropTreeDialog.
    //
    // *nb: all drag-drop and delete functions are removed
    //

  public:
    TTreeNode GetRoot();
    TTreeNode GetSelection();
    TTreeNode GetFirstVisible();

    // Indent level
    //
    uint GetIndent();
    void SetIndent(uint);

    // Image list
    //
    HIMAGELIST GetImageList(TTreeWindow::TImageListType type);
    HIMAGELIST SetImageList(TTreeWindow::TImageListType type,
                            HIMAGELIST newList);

    // Miscellaneous
    //
    uint GetPaneCount();
    uint GetVisibleCount();
    HWND GetEditControl();
    void Update();
    bool EnsureVisible( uint32 paneId );
    HWND EditLabel( uint32 paneId );
    TTreeNode HitTest(TTwHitTestInfo* info);
    TTreeWindow *GetTree();

  private:

    // Should the Apply/Reset button be enabled?
    //
    virtual void EvaluateApplyStatus();
    virtual void EvaluateResetStatus();

    // Do the actual work of inserting the pane.
    //
    TTreeNode InsertPane( TPropTreePane* pane,
                          TTreeNode parent,
                          const char far* text,
                          int image,
                          int selImage );

    // Internal function that switches panes.
    //
    bool DoSwitchPanes( TPropTreePane *oldPane,
                        TPropTreePane *newPane );

    // Internal functions used to find nodes and panes
    //
    TPropTreePane* PaneFromId( uint32 id );
    void FindNodeWithId( uint32 id, TTreeNode parent, TTreeNode& good );

    TPaneInfoArray  paneInfoArray;
    TTreeWindow*    tree;
    TStatic*        placement;
    TStatic*        paneInfo;
    TButton*        Apply;
    TButton*        Reset;

  DECLARE_RESPONSE_TABLE(TPropTreeDialog);
};    //{{TPropTreeDialog}}


//
// The TTreeWindow clone functions are inlined
//

inline TTreeNode
TPropTreeDialog::GetRoot()
{
  return tree->GetRoot();
}

inline TTreeNode
TPropTreeDialog::GetSelection()
{
  return tree->GetSelection();
}


inline TTreeNode
TPropTreeDialog::GetFirstVisible()
{
  return tree->GetFirstVisible();
}


inline uint
TPropTreeDialog::GetIndent()
{
  return tree->GetIndent();
}


inline void
TPropTreeDialog::SetIndent(uint indent)
{
  tree->SetIndent( indent );
}


inline HIMAGELIST
TPropTreeDialog::GetImageList(TTreeWindow::TImageListType type)
{
  return tree->GetImageList( type );
}


inline HIMAGELIST
TPropTreeDialog::SetImageList(TTreeWindow::TImageListType type, HIMAGELIST newList)
{
  return tree->SetImageList( type, newList );
}


inline uint
TPropTreeDialog::GetPaneCount()
{
  return tree->GetItemCount();
}


inline uint
TPropTreeDialog::GetVisibleCount()
{
  return tree->GetVisibleCount();
}


inline HWND
TPropTreeDialog::GetEditControl()
{
  return tree->GetEditControl();
}


inline void
TPropTreeDialog::Update()
{
  tree->Update();
}


inline TTreeNode
TPropTreeDialog::HitTest(TTwHitTestInfo* info)
{
  return TTreeNode( *tree, tree->HitTest( info ) );
}

inline TTreeWindow*
TPropTreeDialog::GetTree()
{
  return tree;
}

#include "proptreepane.h"

#endif  // proptreepage_h sentry.

